#This project was made in December 2020 by Khalil Droubi for UW-Madison, Fall2 2020, CS319 class AND for research purposes. 

#Unless given explicit permission by Khalil, you do NOT have permission to share this code with anybody.

#imports
import math
import pandas as pd
import os
import csv
#import copy
#import sqlite3
import warnings

# Define Molar mass values in dictionary
Oxide_Weight_conversions = {
    "Si": 60.0843, 
    "Ti": 79.8988, 
    "Al": 101.96128, 
    "Cr": 151.99, 
    "Fe": 71.8464, 
    "Mn":70.9374, 
    "Ni": 74.6994, 
    "Mg": 40.3044, 
    "Ca": 56.0794, 
    "Na": 61.97894,
    "K": 94.196}

owc = Oxide_Weight_conversions

def moles_oxygen(dataframe):

    xeno_master_work = {}
    for idx in range(len(dataframe)): 
        xeno_work = {}
        for row in dataframe.columns:

            for element in Oxide_Weight_conversions:
                if element in row:
                    #print(element + ": " + row + ": " + str(xeno.loc[idx, row]))
                    rename = row.split("_")[0] + "_" + element
                    if element == "Si":
                        #print(rename)
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Si"] * 2
                    if element == "Ti":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Ti"] * 2    
                    if element == "Al":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Al"] * 3 
                    if element == "Cr":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Cr"] * 3 
                    if element == "Fe":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Fe"] 
                    if element == "Mn":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Mn"] 
                    if element == "Ni":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Ni"] 
                    if element == "Mg":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Mg"] 
                    if element == "Ca":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Ca"]
                    if element == "Na":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["Na"] 
                    if element == "K":
                        xeno_work[rename] = dataframe.loc[idx, row] / owc["K"]

        name = dataframe.loc[idx, "SampleID"]   
        xeno_master_work[name] = xeno_work
        
    return xeno_master_work

def sum_and_factor(dd): 
    #Input the dict of dicts (dd) from moles_oxygen definition
    mineral_sum = {}
    mineral_factor = {}
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", message="invalid value encountered in double_scalars")
        warnings.filterwarnings("ignore", message="divide by zero encountered in double_scalars")
        for sample in dd:
            sample_sum = {"cpx": 0, "opx": 0, "olv": 0, "gt": 0}
            sample_factor = {}
            for entry in dd[sample]:
                #print(entry)
                #print(df[sample][entry])
                if "cpx" in entry:
                    sample_sum["cpx"] = sample_sum["cpx"] + dd[sample][entry]
                if "opx" in entry:
                    sample_sum["opx"] = sample_sum["opx"] + dd[sample][entry]
                if "olv" in entry:
                    sample_sum["olv"] = sample_sum["olv"] + dd[sample][entry]
                if "gt" in entry:
                    sample_sum["gt"] = sample_sum["gt"] + dd[sample][entry]

            sample_factor["cpx"] = 1 / sample_sum["cpx"] * 6
            sample_factor["opx"] = 1 / sample_sum["opx"] * 6
            sample_factor["olv"] = 1 / sample_sum["olv"] * 4
            sample_factor["gt"]  = 1 / sample_sum["gt"]  * 12

            mineral_sum[sample] = sample_sum
            mineral_factor[sample] = sample_factor
    return mineral_sum, mineral_factor

def formula_units(dd, sample_factor):
    #Input the dict of dicts (dd) from moles_oxygen definition
    sample_formula_units = {}
    
    for sample in dd:
        mineral_formula_units = {}
        
        for entry in dd[sample]:
                   #print(entry)
            if "cpx" in entry:
                working_factor = sample_factor[sample]["cpx"]
            if "opx" in entry:
                working_factor = sample_factor[sample]["opx"]
            if "olv" in entry:
                working_factor = sample_factor[sample]["olv"]
            if "gt" in entry:
                working_factor = sample_factor[sample]["gt"]

            wf = working_factor
            with warnings.catch_warnings():
                warnings.filterwarnings("ignore", message="invalid value encountered in double_scalars")
                warnings.filterwarnings("ignore", message="divide by zero encountered in double_scalars")
                for element in owc:
                    if element in entry:

                        if element == "Si":
                            mineral_formula_units[entry] = dd[sample][entry] * wf / 2

                        if element == "Ti":
                            mineral_formula_units[entry] = dd[sample][entry] * wf / 2

                        if element == "Al":
                            mineral_formula_units[entry] = dd[sample][entry] * wf * 2/3

                        if element == "Cr":
                            mineral_formula_units[entry] = dd[sample][entry] * wf * 2/3

                        if element == "Fe":
                            mineral_formula_units[entry] = dd[sample][entry] * wf

                        if element == "Mn":
                            mineral_formula_units[entry] = dd[sample][entry] * wf

                        if element == "Ni":
                            mineral_formula_units[entry] = dd[sample][entry] * wf

                        if element == "Mg":
                            mineral_formula_units[entry] = dd[sample][entry] * wf

                        if element == "Ca":
                            mineral_formula_units[entry] = dd[sample][entry] * wf

                        if element == "Na":
                            mineral_formula_units[entry] = dd[sample][entry] * wf * 2
                        if element == "K":
                            mineral_formula_units[entry] = dd[sample][entry] * wf * 2
                           
        sample_formula_units[sample] = mineral_formula_units
        
    return sample_formula_units

def formula_units_fast(dataframe):
    dd = moles_oxygen(dataframe)
    #dd = dict of dicts
    x = list(sum_and_factor(dd))
    sample_factor = x[1]
    
    dc = formula_units(dd, sample_factor)
    
    return dc

#Site Occupancy


def site_occupancy(wfu, ferric_choice = 'stoichiometry'):
    '''
    Possible inputs for ferric_choice are:
    (default input) stoichiometry = Calculate Fe3+ in cpx and opx based on cation stoichiometry.
    kd_assumption =  Estimates of maximum Fe3+ as ~25% total iron and ~15% total iron, for cpx and opx, according to the range of data in Dyar et al. (1989) and Luth & Canil (1992).
    [cpx_%Fe3+, opx_%Fe3+] = input a list containing values from 0 to 1 to manually set what percentage of total iron is Fe3+ in each phase.
    '''
    site_occupancy = {}
    fc = ferric_choice
    for sample in wfu:
        min_site = {}
        ms = min_site

        #clinopyroxene and orthopyroxene site occupancy

        ms["cpx_Mg/(Mg+Fe)"] = wfu[sample]["cpx_Mg"] / (wfu[sample]["cpx_Mg"] + wfu[sample]["cpx_Fe"])
        ms["opx_Mg/(Mg+Fe)"] = wfu[sample]["opx_Mg"] / (wfu[sample]["opx_Mg"] + wfu[sample]["opx_Fe"])

        ms["cpx_Mg+Fe_M1"] = 3 - wfu[sample]["cpx_Al"] -  wfu[sample]["cpx_Si"] -  wfu[sample]["cpx_Cr"] - wfu[sample]["cpx_Ti"]
        ms["opx_Mg+Fe_M1"] = 3 - wfu[sample]["opx_Al"] -  wfu[sample]["opx_Si"] -  wfu[sample]["opx_Cr"] - wfu[sample]["opx_Ti"]
        ms["cpx_Mg+Fe_M2"] = 1 - wfu[sample]["cpx_Mn"] -  wfu[sample]["cpx_Ca"] -  wfu[sample]["cpx_Na"] - wfu[sample]["cpx_K"]
        ms["opx_Mg+Fe_M2"] = 1 - wfu[sample]["opx_Mn"] -  wfu[sample]["opx_Ca"] -  wfu[sample]["opx_Na"] - wfu[sample]["opx_K"]

        ms["cpx_Mg_M2"] = ms["cpx_Mg+Fe_M2"] * ms["cpx_Mg/(Mg+Fe)"]
        ms["opx_Mg_M2"] = ms["opx_Mg+Fe_M2"] * ms["opx_Mg/(Mg+Fe)"]

        ms["cpx_a_Mg"] = ms["cpx_Mg_M2"] * (wfu[sample]["cpx_Mg"] - ms["cpx_Mg_M2"])
        ms["opx_a_Mg"] = ms["opx_Mg_M2"] * (wfu[sample]["opx_Mg"] - ms["opx_Mg_M2"])

        ms["opx_XM1_Al_HG"] = wfu[sample]["opx_Al"] / 2
        ms["cpx_XM1_Al_NG"] = (wfu[sample]["cpx_Al"] - wfu[sample]["cpx_Cr"] - (2 * wfu[sample]["cpx_Ti"]) + wfu[sample]["cpx_Na"]) / 2
        ms["opx_XM1_Al_NG"] = (wfu[sample]["opx_Al"] - wfu[sample]["opx_Cr"] - (2 * wfu[sample]["opx_Ti"]) + wfu[sample]["opx_Na"]) / 2

        ms["opx_XM1_Fe,Mg"] = 1 - ms["opx_XM1_Al_NG"] - wfu[sample]["opx_Cr"] - wfu[sample]["opx_Ti"]
        ms["opx_XM1_Mg"] = ms["opx_XM1_Fe,Mg"] * ms["opx_Mg/(Mg+Fe)"]
        ms["opx_XM1_Fe"] = ms["opx_XM1_Fe,Mg"] * (wfu[sample]["opx_Fe"] / (wfu[sample]["opx_Fe"] + wfu[sample]["opx_Mg"]))
        ms["opx_XM1_Cr"] = 1 - ms["opx_XM1_Fe,Mg"] - ms["opx_XM1_Al_NG"] - wfu[sample]["opx_Ti"]

        ms["cpx_Fe#"] = wfu[sample]["cpx_Fe"] / (wfu[sample]["cpx_Fe"] + wfu[sample]["cpx_Mg"])
        ms["opx_Fe#"] = wfu[sample]["opx_Fe"] / (wfu[sample]["opx_Fe"] + wfu[sample]["opx_Mg"])

        ms["cpx_Fe/Mg"] = wfu[sample]["cpx_Fe"] / wfu[sample]["cpx_Mg"]
        ms["opx_Fe/Mg"] = wfu[sample]["opx_Fe"] / wfu[sample]["opx_Mg"]

        # Calculating Fe3+ based on cation stoichiometry
        # I don't agree with this for pyroxenes, but we are going to start out with this calculation, 
        # and figure out how to manually change it later

        # Working on Fe3+ fix, but not finished

        if fc == 'stoichiometry':
            ms["cpx_Fe3+"] = (2 - wfu[sample]["cpx_Si"]) + wfu[sample]["cpx_Na"] - (wfu[sample]["cpx_Al"] - 2 + wfu[sample]["cpx_Si"] + wfu[sample]["cpx_Cr"] + (2 * wfu[sample]["cpx_Ti"]))
            ms["opx_Fe3+"] = (2 - wfu[sample]["opx_Si"]) + wfu[sample]["opx_Na"] - (wfu[sample]["opx_Al"] - 2 + wfu[sample]["opx_Si"] + wfu[sample]["opx_Cr"] + (2 * wfu[sample]["opx_Ti"]))

        if fc == 'kd_assumption':
            ## Estimates of maximum Fe3+ as ~25% total iron and ~15% total iron, for clinopyroxene and orthopyroxene,
            # according to the range of data in Dyar et al. (1989) and Luth & Canil (1992).
            ms["cpx_Fe3+"] = wfu[sample]["cpx_Fe"] * 0.25
            ms["opx_Fe3+"] = wfu[sample]["opx_Fe"] * 0.15
        # Future if statements should allow for manual set of cpx or opx Fe3+ = X% of Fe(total)
        if type(fc) == "list":
            ms["cpx_Fe3+"] = wfu[sample]["cpx_Fe"] * fc[0]
            ms["opx_Fe3+"] = wfu[sample]["opx_Fe"] * fc[1]
        else:
            pass
            #ms["cpx_Fe3+"] = pass
            #ms["opx_Fe3+"] = pass

        ms["cpx_%Fe3+"] = 100 * ms["cpx_Fe3+"] / wfu[sample]["cpx_Fe"]
        ms["opx_%Fe3+"] = 100 * ms["opx_Fe3+"] / wfu[sample]["opx_Fe"]   

        # We want to be able to  change the %Fe3+ to between 0 and 24%, realistically.

        ms["cpx_Fe2+/Mg"] = (wfu[sample]["cpx_Fe"] - ms["cpx_Fe3+"]) / wfu[sample]["cpx_Mg"]
        ms["opx_Fe2+/Mg"] = (wfu[sample]["opx_Fe"] - ms["opx_Fe3+"]) / wfu[sample]["opx_Mg"]
        ms["cpx_Jadeite"] = wfu[sample]["cpx_Na"] - wfu[sample]["cpx_Cr"] - ms["cpx_Fe3+"] - (2 * wfu[sample]["cpx_Ti"])
        ms["opx_Jadeite"] = wfu[sample]["opx_Na"] - wfu[sample]["opx_Cr"] - ms["opx_Fe3+"] - (2 * wfu[sample]["opx_Ti"])

        ms["cpx_XAl_M1"] = (wfu[sample]["cpx_Al"] + abs(ms["cpx_Jadeite"])) / 2
        ms["opx_XAl_M1"] = (wfu[sample]["opx_Al"] + ms["opx_Jadeite"]) / 2

        ms["cpx_Xmf_M1"] = 1 - ((wfu[sample]["cpx_Al"] + ms["cpx_Jadeite"]) / 2) -  wfu[sample]["cpx_Cr"] - wfu[sample]["cpx_Ti"]
        ms["opx_Xmf_M1"] = 1 - ms["opx_XAl_M1"] - wfu[sample]["opx_Cr"] - ms["opx_Fe3+"] - wfu[sample]["opx_Ti"]
        ms["cpx_Xmf_M2"] = 1 - wfu[sample]["cpx_Ca"] - wfu[sample]["cpx_Na"] - wfu[sample]["cpx_Mn"]
        ms["opx_Xmf_M2"] = 1 - wfu[sample]["opx_Ca"] - wfu[sample]["opx_Na"] - wfu[sample]["opx_Mn"]

        ms["cpx_Xmg_M1"] = ms["cpx_Xmf_M1"] * ms["cpx_Mg/(Mg+Fe)"]
        ms["opx_Xmg_M1"] = ms["opx_Mg/(Mg+Fe)"] * (1 - ((wfu[sample]["opx_Al"] + ms["opx_Jadeite"]) / 2) - wfu[sample]["opx_Cr"] - wfu[sample]["opx_Ti"])
        ms["cpx_Xmg_M2"] = ms["cpx_Xmf_M2"] * ms["cpx_Mg/(Mg+Fe)"]
        ms["opx_Xmg_M2"] = ms["opx_Mg/(Mg+Fe)"] * (1 - wfu[sample]["opx_Ca"] - wfu[sample]["opx_Na"] - wfu[sample]["opx_Mn"])
        ms["cpx_Xfe_M1"] = (ms["cpx_Xmg_M1"] - (ms["cpx_Xmg_M1"] * ms["cpx_Mg/(Mg+Fe)"])) / ms["cpx_Mg/(Mg+Fe)"]
        ms["opx_Xfe_M1"] = (ms["opx_Xmg_M1"] - (ms["opx_Xmg_M1"] * ms["opx_Mg/(Mg+Fe)"])) / ms["opx_Mg/(Mg+Fe)"]

        ms["cpx_a(en)"] = (1 - wfu[sample]["cpx_Ca"] - wfu[sample]["cpx_Na"] - wfu[sample]["cpx_K"]) * ( 1 - (0.5 * (wfu[sample]["cpx_Al"] + wfu[sample]["cpx_Cr"] + wfu[sample]["cpx_Na"] + wfu[sample]["cpx_K"])))                                                                                               

        #garnet site occupancy

        ms["gt_XCa"] = wfu[sample]["gt_Ca"] / (wfu[sample]["gt_Ca"] + wfu[sample]["gt_Mg"] + wfu[sample]["gt_Fe"] + wfu[sample]["gt_Mn"])
        ms["gt_XAl"] = wfu[sample]["gt_Al"] / (wfu[sample]["gt_Al"] + wfu[sample]["gt_Cr"])
        ms["gt_XCr"] = wfu[sample]["gt_Cr"] / (wfu[sample]["gt_Al"] + wfu[sample]["gt_Cr"])
        ms["gt_Fe#"] = wfu[sample]["gt_Fe"] / (wfu[sample]["gt_Fe"] + wfu[sample]["gt_Mg"])
        ms["gt_XFe"] = wfu[sample]["gt_Fe"] / (wfu[sample]["gt_Fe"] + wfu[sample]["gt_Mn"] + wfu[sample]["gt_Mg"] + wfu[sample]["gt_Ca"])
        ms["gt_XMg"] = wfu[sample]["gt_Mg"] / (wfu[sample]["gt_Fe"] + wfu[sample]["gt_Mn"] + wfu[sample]["gt_Mg"] + wfu[sample]["gt_Ca"])
        ms["gt_XMn"] = wfu[sample]["gt_Mn"] / (wfu[sample]["gt_Fe"] + wfu[sample]["gt_Mn"] + wfu[sample]["gt_Mg"] + wfu[sample]["gt_Ca"])
        ms["gt_Fe/Mg"] = wfu[sample]["gt_Fe"] / wfu[sample]["gt_Mg"]

        #olivine site occupancy

        ms["olv_Fe#"] = wfu[sample]["olv_Fe"] / (wfu[sample]["olv_Fe"] + wfu[sample]["olv_Mg"])
        ms["olv_Fe/Mg"] = wfu[sample]["olv_Fe"] / wfu[sample]["olv_Mg"]


        site_occupancy[sample] = min_site

    return site_occupancy

# Beyer site occupancy

# note that Fe3+ will affect these results, so need to figure out how to manually change Fe3+ in wfu

def beyer_paramaters(wfu, so):
    '''
    Calculated parameters from the Beyer 2016 ResearchGate spreadsheet:

    https://www.researchgate.net/publication/306038208_Update_2_Beyer_et_al_2015_grt-cpx_barometer_for_eclogites_Iterative_calculation_of_P_and_T
    '''
    site_occupancy = {}

    for sample in wfu:
        min_site = {}
        ms = min_site
        
        #garnet site occupancy
        
        ms['gt_normFe'] = 3 * wfu[sample]['gt_Fe'] / (wfu[sample]['gt_Fe'] + wfu[sample]['gt_Mg'] + wfu[sample]['gt_Ca'])
        ms['gt_normMg'] = 3 * wfu[sample]['gt_Mg'] / (wfu[sample]['gt_Fe'] + wfu[sample]['gt_Mg'] + wfu[sample]['gt_Ca'])
        ms['gt_normCa'] = 3 * wfu[sample]['gt_Ca'] / (wfu[sample]['gt_Fe'] + wfu[sample]['gt_Mg'] + wfu[sample]['gt_Ca'])
        ms['gt_catTotal'] = ms['gt_normFe'] + ms['gt_normMg'] + ms['gt_normCa']
        
        ms['gt_XFe'] = wfu[sample]['gt_Fe'] / (wfu[sample]['gt_Fe'] + wfu[sample]['gt_Mg'] + wfu[sample]['gt_Ca'])
        ms['gt_XMg'] = wfu[sample]['gt_Mg'] / (wfu[sample]['gt_Fe'] + wfu[sample]['gt_Mg'] + wfu[sample]['gt_Ca'])
        ms['gt_XCa'] = wfu[sample]['gt_Ca'] / (wfu[sample]['gt_Fe'] + wfu[sample]['gt_Mg'] + wfu[sample]['gt_Ca'])

    
        #clinopyroxene site occupancy
        
        ms['cpx_Al_IV'] = 2 - wfu[sample]['cpx_Si']
        ms['cpx_Fe+Mg_M1'] = 1 - (wfu[sample]['cpx_Al'] - ms['cpx_Al_IV']) - so[sample]['cpx_Fe3+'] - wfu[sample]['cpx_Ti'] 
        ms['cpx_XFe2+_M1'] = wfu[sample]['cpx_Fe'] / (wfu[sample]['cpx_Fe'] + wfu[sample]['cpx_Mg']) * ms['cpx_Fe+Mg_M1']
        ms['cpx_XFe3+_M1'] = so[sample]['cpx_Fe3+']
        ms['cpx_XMg_M1'] = wfu[sample]['cpx_Mg'] / (wfu[sample]['cpx_Fe'] + wfu[sample]['cpx_Mg']) * ms['cpx_Fe+Mg_M1']
        ms['cpx_XAl_M1'] = wfu[sample]['cpx_Al'] - ms['cpx_Al_IV']
        
        ms['cpx_Fe+Mg_M2'] = wfu[sample]['cpx_Fe'] + wfu[sample]['cpx_Mg'] - ms['cpx_Fe+Mg_M1']
        ms['cpx_XCa_M2'] = wfu[sample]['cpx_Ca']
        ms['cpx_XNa_M2'] = wfu[sample]['cpx_Na']
        ms['cpx_XFe_M2'] = wfu[sample]['cpx_Fe'] / (wfu[sample]['cpx_Fe'] + wfu[sample]['cpx_Mg']) * ms['cpx_Fe+Mg_M2']
        ms['cpx_XMg_M2'] = wfu[sample]['cpx_Mg'] / (wfu[sample]['cpx_Fe'] + wfu[sample]['cpx_Mg']) * ms['cpx_Fe+Mg_M2']
        ms['cpx_XAl_Esk_M2'] = (wfu[sample]['cpx_Al'] - wfu[sample]['cpx_Na']) - (2 *(2 - wfu[sample]['cpx_Si']))
            #XAl_Esk is weird...
        ms['cpx_Ca+Na_M2'] = ms['cpx_XCa_M2'] + ms['cpx_XNa_M2']
        
        site_occupancy[sample] = min_site

    return site_occupancy       

def initial_parameters(input_csv, ferric_choice):
    '''
    Input your .csv file of xenolith data from the provided template, and this function will output a dictionary containing wfu, so, and bp.

    After invoking function, you should pull out wfu, so, and bp as follows:
    [start code]

    parameters_list = tb_param.initial_parameters('xenolith_small.csv')
    wfu = parameters_list['wfu']
    so = parameters_list['so']
    bp = parameters_list['bp']
    '''
    parameters_list = {}
    
    xeno = pd.read_csv(input_csv)

    working_form_units = formula_units_fast(xeno)

# These are the 3 dictionaries that contain factors for P-T calculations

    wfu = working_form_units
    so = site_occupancy(wfu, ferric_choice)
    bp = beyer_paramaters(wfu, so)


    parameters_list['wfu'] = wfu
    parameters_list['so'] = so
    parameters_list['bp'] = bp

    return parameters_list

def new_template():
    '''
Makes a new xenolith data template .csv file for you to input your xenolith data.
    '''

    if os.path.isfile('xenolith_dataTemplate.csv'):
        print('Data template file already exists in the current directory.')
    else:
        template = [{'Locality': 'Test 1', 'Reference': 'test study', 'SampleID': 'test1', 'cpx_SiO2': '50.58964', 'cpx_TiO2': '0.588516', 'cpx_Al2O3': '8.649199', 'cpx_Cr2O3': '0.07089805', 'cpx_FeO': '3.376068', 'cpx_MnO': '0.03865455', 'cpx_NiO': '0', 'cpx_MgO': '13.091855', 'cpx_CaO': '19.602325', 'cpx_Na2O': '2.4164125', 'cpx_K2O': '0.00064325', 'cpx_Total': '98.42', 'opx_SiO2': '53.11461304', 'opx_TiO2': '0.120747652', 'opx_Al2O3': '5.781704348', 'opx_Cr2O3': '0.038188087', 'opx_FeO': '7.601890435', 'opx_MnO': '0.063495348', 'opx_NiO': '0', 'opx_MgO': '31.73361739', 'opx_CaO': '0.252286652', 'opx_Na2O': '0.041200913', 'opx_K2O': '6.15652E-05', 'opx_Total': '98.75', 'gt_SiO2': '41.11361667', 'gt_TiO2': '0.090340729', 'gt_Al2O3': '24.25045208', 'gt_Cr2O3': '0.058934667', 'gt_FeO': '10.53211667', 'gt_MnO': '0.224152396', 'gt_NiO': '0', 'gt_MgO': '19.20767083', 'gt_CaO': '4.561709375', 'gt_Na2O': '0.02429225', 'gt_K2O': '0', 'gt_Total': '100.06', 'olv_SiO2': '0', 'olv_TiO2': '0', 'olv_Al2O3': '0', 'olv_Cr2O3': '0', 'olv_FeO': '0', 'olv_MnO': '0', 'olv_NiO': '0', 'olv_MgO': '0', 'olv_CaO': '0', 'olv_Na2O': '0', 'olv_K2O': '0', 'olv_Total': '0'},
                    {'Locality': 'Test 2', 'Reference': 'test study', 'SampleID': 'test2', 'cpx_SiO2': '50.93', 'cpx_TiO2': '0.68', 'cpx_Al2O3': '8.86', 'cpx_Cr2O3': '0.02', 'cpx_FeO': '5.45', 'cpx_MnO': '0.033', 'cpx_NiO': '0', 'cpx_MgO': '12.58', 'cpx_CaO': '17.64', 'cpx_Na2O': '3.02', 'cpx_K2O': '0', 'cpx_Total': '99.21', 'opx_SiO2': '52.92', 'opx_TiO2': '0.16', 'opx_Al2O3': '5.14', 'opx_Cr2O3': '0.01', 'opx_FeO': '11.45', 'opx_MnO': '0.09', 'opx_NiO': '0', 'opx_MgO': '29.13', 'opx_CaO': '0.55', 'opx_Na2O': '0.11', 'opx_K2O': '0', 'opx_Total': '99.56', 'gt_SiO2': '40.49', 'gt_TiO2': '0.14', 'gt_Al2O3': '23.85', 'gt_Cr2O3': '0.01', 'gt_FeO': '15.25', 'gt_MnO': '0.33', 'gt_NiO': '0', 'gt_MgO': '16.28', 'gt_CaO': '4.37', 'gt_Na2O': '0.03', 'gt_K2O': '0', 'gt_Total': '100.75', 'olv_SiO2': '0', 'olv_TiO2': '0', 'olv_Al2O3': '0', 'olv_Cr2O3': '0', 'olv_FeO': '0', 'olv_MnO': '0', 'olv_NiO': '0', 'olv_MgO': '0', 'olv_CaO': '0', 'olv_Na2O': '0', 'olv_K2O': '0', 'olv_Total': '0'}]
        dff = pd.DataFrame(template)
        dff.to_csv('xenolith_dataTemplate.csv', index = False)
        print('New template title "xenolith_dataTemplate.csv" should be in current directory!')
    return None

#RUNNING CODE 



'''
Mathematical functions for calculating pressure and temperature given the mineral chemistries of garnet, clinopyroxene, and orthopyroxene.

Made by Khalil Droubi, January 2021

Needs to be used with the tb_parameter module.
Before using any of these functions, type:

[start of code]
import tb_parameters as tb_param
import thermobarometers as tb

parameters_list = tb_param.initial_parameters({Your csv file here})

wfu = parameters_list['wfu'] // Weighted formula units for minerals
so = parameters_list['so']   // Site occupancies
bp = parameters_list['bp']   // Separately calculated parameters for the Beyer barometer
[end of code]

Ignore the warnings about double scalars for now.
'''

#imports
import math
import pandas as pd
#import os
#import csv
#import copy
#import matplotlib

# Thermometers:
# Wells 1977 (OLD)
# EG 1979 (OLD)
# Krogh 88 (OLD)

# EG79 w/Fe3+ (PASSED) gt-cpx
# K88 w/Fe3+ (PASSED) gt-cpx
# TBKN 1989 (PASSED) cpx-opx
# T Ca in Opx (PASSED) opx
# Nimis & Taylor (May fail on occasion- math domain error related to certain samples) cpx
# Nakamara(2009) (kind of PASSED? Need to test further) gt-cpx

# Krogh-Ravna(2000) (maybe in future?)
# Putirka (2008) (maybe in future?)

# Barometers
# NG85 (PASSED) gt-opx?
# PBKN (PASSED) gt-opx
# Beyer(2016) (kind of PASSED? Need to test further) gt-cpx

#thermometers

def tbkn(sample_wfu, sample_so, pressure):
    '''
    Clinopyroxene-orthopyroxene Mg-Fe exchange thermometer from Brey and Kohler, 1990

    Brey, G. P., & Köhler, T. (1990). Geothermobarometry in four-phase lherzolites II. New
thermobarometers, and practical assessment of existing thermobarometers. Journal of Petrology,
31(6), 1353-1378.
    '''
    
    # Input T = degrees celsius and P = kilobars
    p = pressure
    
    t25 = (1 - (sample_wfu['cpx_Ca'] / (1 - sample_wfu['cpx_Na']))) / (1 - (sample_wfu['opx_Ca'] / (1 - sample_wfu['opx_Na'])))
    
    numerator = 23664 + (p * (24.9 + (126.3 * sample_so['cpx_Fe#'])))
    denominator = 13.38 + (math.log(t25)**2) + (11.59 * sample_so['opx_Fe#'])
    
    temp_k = (numerator / denominator) #in kelvin
    
    temp_c = temp_k - 273.15 # in celsius
    
    #print ("t25 = " + str(t25))
    
    return temp_c


def T_Ca_in_Opx(sample_wfu, sample_so, pressure):
    '''
    Ca-in-orthopyroxene thermometer from Brey and Kohler, 1990

    Brey, G. P., & Köhler, T. (1990). Geothermobarometry in four-phase lherzolites II. New
thermobarometers, and practical assessment of existing thermobarometers. Journal of Petrology,
31(6), 1353-1378.
    '''
     
    # Input T = degrees celsius and P = kilobars
    p = pressure
    
    temp_k = (6425 + 26.4 * p) / (-math.log(sample_wfu['opx_Ca']) + 1.843)   #in kelvin

    temp_c = temp_k - 273.15    #in celsius
    return temp_c    

def T_K88_wFe3(sample_wfu, sample_so, pressure):
    '''
    Garnet-clinopyroxene Mg-Fe exchange thermometer modified from Krogh, 1988

    Modifications made to correct for Fe3+ in clinopyroxene

    Krogh, E. J. (1988). The garnet-clinopyroxene Fe-Mg geothermometer—a reinterpretation of existing
experimental data. Contributions to Mineralogy and Petrology, 99(1), 44-48.
    '''
    # Input T = degrees celsius and P = kilobars
    p = pressure
    
    temp_k = ( (-6173 * (sample_so['gt_XCa']**2)) + (6731 * sample_so['gt_XCa']) + 1879 + (10 * p) ) / (math.log(sample_so['gt_Fe/Mg'] / sample_so["cpx_Fe2+/Mg"]) + 1.393) #in kelvin
    temp_c = temp_k - 273.15    #in celsius
    
    return temp_c  

def T_EG79_wFe3(sample_wfu, sample_so, pressure):
    '''
    Garnet-clinopyroxene Mg-Fe exchange thermometer modified from Ellis and Green, 1979

    Modifications made to correct for Fe3+ in clinopyroxene

    Ellis, D. J., & Green, D. H. (1979). An experimental study of the effect of Ca upon garnet-clinopyroxene
Fe-Mg exchange equilibria. Contributions to Mineralogy and Petrology, 71(1), 13-22.
    '''
    # Input T = degrees celsius and P = kilobars
    p = pressure
    
    temp_k = (3104 * (sample_wfu['gt_Ca'] / (sample_wfu['gt_Ca'] + sample_wfu['gt_Fe'] + sample_wfu['gt_Mg'])) + 3030 + (10.86 * p) ) / (math.log( (sample_wfu['gt_Fe'] / sample_wfu['gt_Mg']) / ( (sample_wfu['cpx_Fe'] - sample_so['cpx_Fe3+']) / sample_wfu['cpx_Mg'])) + 1.9034) #in kelvin
    
    temp_c = temp_k - 273.15    #in celsius
    
    return temp_c

def T_Nimis_and_Taylor(sample_wfu, sample_so, pressure):
    '''
    Clinopyroxene thermometer from Nakamura, 2009

    Taylor, W. R. (1998). An experimental test of some geothermometer and geobaro-meter formulations for
upper mantle peridotites with application to the ther-mobarometry of fertile lherzolite and garnet
websterite. Neues Jahrbuch für Mineralogie-Abhandlungen, 381-408.

    Nimis, P., & Grütter, H. (2010). Internally consistent geothermometers for garnet peridotites and
pyroxenites. Contributions to Mineralogy and Petrology, 159(3), 411-427.
    '''
    # Input T = degrees celsius and P = kilobars
    p = pressure
    
    temp_k = (23116 + 39.28 * p) / (13.25 + 15.35 * sample_wfu['cpx_Ti'] + 4.5 * sample_wfu['cpx_Fe'] - (1.55 * (sample_wfu['cpx_Al'] + sample_wfu['cpx_Cr'] - sample_wfu['cpx_Na'] - sample_wfu['cpx_K'])) + (math.log(sample_so["cpx_a(en)"])**2) ) 
    
    temp_c = temp_k - 273.15    #in celsius
    
    return temp_c

def T_N09(sample_wfu, sample_so, pressure):
    '''
    Garnet-clinopyroxene thermometer from Nakamura, 2009


    Nakamura, D. (2009). A new formulation of garnet–clinopyroxene geothermometer based on
accumulation and statistical analysis of a large experimental data set. Journal of Metamorphic
Geology, 27(7), 495-508.
    '''
    # Input T = degrees celsius and P = kilobars
    p = pressure
    
    Xprp = sample_so["gt_XMg"]
    Xalm = sample_so["gt_XFe"]
    Xsps = sample_so["gt_XMn"]
    Xgrs = sample_so["gt_XCa"]
    cpx_Xmg = sample_wfu['cpx_Mg'] / (sample_wfu['cpx_Al'] + sample_wfu['cpx_Fe'] + sample_wfu['cpx_Mg'])
    cpx_Xfe = sample_wfu['cpx_Fe'] / (sample_wfu['cpx_Al'] + sample_wfu['cpx_Fe'] + sample_wfu['cpx_Mg'])
    A = 0.5 * Xgrs * (Xprp - Xalm - Xsps)
    B = 0.5 * Xgrs * (Xprp - Xalm + Xsps)
    C = 0.5 * (Xgrs + Xsps) * (Xprp - Xalm)
    Kd = sample_so["gt_Fe/Mg"] / sample_so["cpx_Fe/Mg"]
    
    
    v1 = 2784 + 14.52 * p + (2601 + 1.44 * p) * (2 * Xgrs * Xprp - A)
    v2 = (1183 + 6.98 * p) * (Xgrs ** 2 - A)
    v3 = 105 * (2 * Xgrs * Xalm + B)
    v4 = (814.6 + 3.61 * p) * (Xgrs**2 + B)
    v5 = (254.6 + 8.42 * p) * (2 * Xprp * Xalm - Xalm**2 + C)
    v6 = 83.6 * (Xprp**2 - 2 * Xprp * Xalm + C)
    v7 =  1388 * Xsps - 462 * (cpx_Xmg - cpx_Xfe)
    v_numerator = v1 + v2 - v3 + v4 - v5 - v6 + v7
    u1 = math.log(Kd) + 1.431 + 0.695 * (2 * Xgrs * Xprp + Xgrs**2 - 2 * A)
    u2 = 0.203 * (Xgrs**2 - 2 * Xgrs * Xalm) + 0.922 * Xsps
    
    temp_k = v_numerator / (u1 + u2) # in kelvin
    temp_c = temp_k - 273.15    #in celsius
    
    return temp_c
                                    
#barometers                                    
                                    
def pbkn(sample_wfu, sample_so, sample_bp, temperature):
    '''
    Garnet-orthopyroxene baromometer from Brey and Kohler, 1990

    Brey, G. P., & Köhler, T. (1990). Geothermobarometry in four-phase lherzolites II. New
thermobarometers, and practical assessment of existing thermobarometers. Journal of Petrology,
31(6), 1353-1378.
    '''
    # Input T = degrees celsius and P = kilobars
    
    t = temperature + 273 # in kelvin
    
    Kd = ( ((1 - sample_so['gt_XCa']) ** 3) * (sample_so['gt_XAl'] ** 2) ) / (sample_so['opx_Xmf_M1'] * (sample_so['opx_Xmf_M2'] ** 2) * sample_so['opx_XAl_M1'])
    #Kd PASSED
    
    c1 = ((-8.3143) * t * math.log(Kd)) - 5510 + (88.91 * t) - (19 * (t**1.2)) + (3 * (sample_so['gt_XCa']**2) * 82458) + (sample_so['opx_Xmg_M1'] * sample_so['opx_Xfe_M1'] * (80942 - (46.7 * t))) - (3 * sample_so['gt_XFe'] * sample_so['gt_XCa'] * 17793) - (sample_so['gt_XCa'] * sample_so['gt_XCr'] * ((1.164 * 10**6) - (420.4 * t))) - (sample_so['gt_XFe'] * sample_so['gt_XCr'] * (((-1.25) * 10**6) + (565 * t)))      
    #c1 PASSED
    
    c2 = -0.832 - (8.78 * 10**(-5) * (t - 298)) + (3 * sample_so['gt_XCa']**2 * 3.305) - (sample_so['gt_XCa'] * sample_so['gt_XCr'] * 13.45) + (sample_so['gt_XFe'] * sample_so['gt_XCr'] * 10.5)
    #c2 PASSED
    
    pressure = (-1*c2 - math.sqrt(c2**2 + (4 * 16.6 * 10**(-4) * c1 / 1000))) / (2 * 16.6 * 10**(-4))
    #slightly off by 0.005 Kbar
    
    
    #print("Kd = " + str(Kd))
    #print('c1 = ' + str(c1))
    #print('c2 = ' + str(c2))
    #print(pressure)
    
    return pressure

def NG85(sample_wfu, sample_so, sample_bp, temperature):
    '''
    Garnet-orthopyroxene baromometer from Nickel and Green, 1985


    Nickel, K. G., & Green, D. H. (1985). Empirical geothermobarometry for garnet peridotites and
implications for the nature of the lithosphere, kimberlites and diamonds. Earth and Planetary
Science Letters, 73(1), 158-170.
    '''
    # Input T = degrees celsius and P = kilobars
    
    t = temperature + 273 # in kelvin
    
    NG1 = (-1) / (183.3 + 178.98 * sample_so['opx_XM1_Al_NG'] * (1 - sample_so['opx_XM1_Al_NG']))
    #NG1 PASSED
    
    NG2 = (-1.98717 * t) * math.log( ( ((1 - sample_so['gt_XCa']) ** 3) * (sample_so['gt_XAl'] ** 2) ) / (sample_so['opx_XM1_Fe,Mg'] * (sample_so['opx_Mg+Fe_M2']**2) * sample_so['opx_XM1_Al_NG']) ) - (9000 * (sample_so['gt_XCa']**2)) - 3400 * (2 * (sample_so['gt_XCr']**2) - (sample_so['opx_XM1_Mg'] * sample_so['opx_XM1_Cr'])) - (sample_so['gt_XCa'] * sample_so['gt_XCr'] * (90853 - (52.1 * t))) - (7590 * sample_so['gt_XFe'] * sample_so['gt_XCa']) + (5157 * sample_so['opx_XM1_Mg'] * sample_so['opx_XM1_Fe']) + 6047 - (3.23 * t)      
    #NG2 PASSED
    
    pressure = NG1 * NG2
    
    #print(NG1)
    #print(NG2)
    
    return pressure


def P_Beyer(sample_wfu, sample_so, sample_bp, temperature):
    '''
    Garnet-clinopyroxene "eclogite" barometer from Beyer et al., 2015

    Beyer, C., Frost, D. J., & Miyajima, N. (2015). Experimental calibration of a garnet–clinopyroxene
geobarometer for mantle eclogites. Contributions to Mineralogy and Petrology, 169(2), 1-21.
    '''


    # Input T = degrees celsius and P = kilobars
    
    t = temperature + 273 # in kelvin
    
 # Interaction Parameters
    #garnet Margules parameters, values from Ganguly et al., 1996, with pressure dependent terms equal to zero
    gt_W_CaMg = 21627 - (t * 5.78)
    gt_W_MgCa = 9834 - (t * 5.78)
    gt_W_CaFe = 873 - (1.69 * t)
    gt_W_FeCa = 6773 - (1.69 * t)
    gt_W_MgFe = 2117
    gt_W_FeMg = 695

    #gt_grs_dodec_RTg = ((0.5 * gt_W_CaMg * sample_bp['gt_XMg'] * (1 - sample_bp['gt_XCa'] + sample_bp['gt_XMg'] + 2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XMg'] - 1)) )  + (0.5 * gt_W_MgCa * sample_bp['gt_XMg'] * ( 1 - sample_bp['gt_XCa'] - sample_bp['gt_XMg'] - (2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XMg'] - 1)) + 0.5 * gt_W_CaFe * sample_bp['gt_XFe'] * (1 - sample_bp['gt_XCa'] + sample_bp['gt_XFe'] + (2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XFe'] -1))) + (0.5 * gt_W_FeCa * sample_bp['gt_XFe'] * (1 - sample_bp['gt_XCa'] - sample_bp['gt_XFe'] - (2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XFe'] - 1)))) + (gt_W_MgFe * sample_bp['gt_XMg'] * sample_bp['gt_XFe'] * (sample_bp['gt_XFe'] - sample_bp['gt_XMg'] - 0.5)) + (gt_W_FeMg * sample_bp['gt_XFe'] * sample_bp['gt_XMg'] * (sample_bp['gt_XFe'] - sample_bp['gt_XMg'] - 0.5)) + (  (((gt_W_MgCa - gt_W_CaMg) + (gt_W_CaFe - gt_W_FeCa) + (gt_W_FeMg - gt_W_MgFe)) / 2) * sample_bp['gt_XMg'] * sample_bp['gt_XFe'] * (1 - 2 * sample_bp['gt_XCa']) ) )) ) 
    
    p1 = 0.5 * gt_W_CaMg * sample_bp['gt_XMg'] * (1 - sample_bp['gt_XCa'] + sample_bp['gt_XMg'] + 2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XMg'] - 1))             
    p2 = 0.5 * gt_W_MgCa * sample_bp['gt_XMg'] * (1 - sample_bp['gt_XCa'] - sample_bp['gt_XMg'] - 2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XMg'] - 1))                                                                                                                               
    p3 = 0.5 * gt_W_CaFe * sample_bp['gt_XFe'] * (1 - sample_bp['gt_XCa'] + sample_bp['gt_XFe'] + 2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XFe'] -1))                                        
    p4 = 0.5 * gt_W_FeCa * sample_bp['gt_XFe'] * (1 - sample_bp['gt_XCa'] - sample_bp['gt_XFe'] - 2 * sample_bp['gt_XCa'] * (sample_bp['gt_XCa'] - sample_bp['gt_XFe'] - 1))                                        
    p5 =       gt_W_MgFe * sample_bp['gt_XMg'] * sample_bp['gt_XFe'] * (sample_bp['gt_XFe'] - sample_bp['gt_XMg'] - 0.5)                                        
    p6 =       gt_W_FeMg * sample_bp['gt_XFe'] * sample_bp['gt_XMg'] * (sample_bp['gt_XFe'] - sample_bp['gt_XMg'] - 0.5)
    p7 =  (((gt_W_MgCa - gt_W_CaMg) + (gt_W_CaFe - gt_W_FeCa) + (gt_W_FeMg - gt_W_MgFe)) / 2) * sample_bp['gt_XMg'] * sample_bp['gt_XFe'] * (1 - 2 * sample_bp['gt_XCa'])   
    
    gt_grs_dodec_RTg = p1 + p2 + p3 + p4 + p5 + p6 + p7
    gt_grs_RTLN_X3 = 8.314 * t * math.log(sample_bp['gt_XCa']**3)
    gt_grs_Rtlna_CaAl = 3 * gt_grs_dodec_RTg + gt_grs_RTLN_X3
    
    u1 = 0.5 * gt_W_MgCa * sample_bp['gt_XCa'] * (1 - sample_bp['gt_XMg'] + sample_bp['gt_XCa'] + 2 * sample_bp['gt_XMg'] * (sample_bp['gt_XMg'] - sample_bp['gt_XCa'] - 1))             
    u2 = 0.5 * gt_W_CaMg * sample_bp['gt_XCa'] * (1 - sample_bp['gt_XMg'] - sample_bp['gt_XCa'] - 2 * sample_bp['gt_XMg'] * (sample_bp['gt_XMg'] - sample_bp['gt_XCa'] - 1))                                                                                                                               
    u3 = 0.5 * gt_W_MgFe * sample_bp['gt_XFe'] * (1 - sample_bp['gt_XMg'] + sample_bp['gt_XFe'] + 2 * sample_bp['gt_XMg'] * (sample_bp['gt_XMg'] - sample_bp['gt_XFe'] -1))                                        
    u4 = 0.5 * gt_W_FeMg * sample_bp['gt_XFe'] * (1 - sample_bp['gt_XMg'] - sample_bp['gt_XFe'] - 2 * sample_bp['gt_XMg'] * (sample_bp['gt_XMg'] - sample_bp['gt_XFe'] - 1))                                        
    u5 =       gt_W_CaFe * sample_bp['gt_XCa'] * sample_bp['gt_XFe'] * (sample_bp['gt_XCa'] - sample_bp['gt_XFe'] - 0.5)                                        
    u6 =       gt_W_FeCa * sample_bp['gt_XFe'] * sample_bp['gt_XCa'] * (sample_bp['gt_XFe'] - sample_bp['gt_XCa'] - 0.5)
    u7 =  (((gt_W_MgCa - gt_W_CaMg) + (gt_W_CaFe - gt_W_FeCa) + (gt_W_FeMg - gt_W_MgFe)) / 2) * sample_bp['gt_XCa'] * sample_bp['gt_XFe'] * (1 - 2 * sample_bp['gt_XMg'])   
    
    gt_pyp_dodec_RTg = u1 + u2 + u3 + u4 + u5 + u6 + u7
    gt_pyp_RTLN_X3Y2 = 8.314 * t * math.log(sample_bp['gt_XMg']**3)
    gt_pyp_Rtlna_MgAl = 3 * gt_pyp_dodec_RTg + gt_pyp_RTLN_X3Y2
    
        #clinopyroxene Margules parameters, values from Ganguly et al., 1996
    
    cpx_W_MgFe_M1 = 0
    cpx_W_MgAl_M1 = 4849
    cpx_W_FeAl_M1 = 3573
    cpx_W_CaNa_M2 = 11567
    cpx_W_MgNa_M2 = -35873
    cpx_W_FeNa_M2 = 59319
    cpx_W_CaMg_M2 = 24920
    cpx_W_CaFe_M2 = -49275
    cpx_W_FeMg_M2 = 0
    cpx_W_AlSi_M2 = 7038
    
    cpx_diop_RTln = 8.314 * t * math.log(sample_bp['cpx_XMg_M1'] * sample_bp['cpx_XCa_M2'] * (sample_wfu['cpx_Si'] / 2)**2 )
    
    v1 = (sample_bp['cpx_XMg_M2'] ** 2) * cpx_W_CaMg_M2 + (sample_bp['cpx_XNa_M2'] ** 2) * cpx_W_CaNa_M2 + (sample_bp['cpx_XFe_M2'] ** 2) * cpx_W_CaFe_M2
    v2 = sample_bp['cpx_XMg_M2'] * sample_bp['cpx_XFe_M2'] * (cpx_W_CaMg_M2 + cpx_W_CaFe_M2 -  cpx_W_FeMg_M2) + sample_bp['cpx_XMg_M2'] * sample_bp['cpx_XNa_M2'] * (cpx_W_CaMg_M2 + cpx_W_CaNa_M2 - cpx_W_MgNa_M2) + sample_bp['cpx_XNa_M2'] * sample_bp['cpx_XFe_M2'] * (cpx_W_CaNa_M2 + cpx_W_CaFe_M2 -  cpx_W_FeNa_M2)
    cpx_diop_RTlna_M2 = v1 + v2
    
    cpx_diop_RTlna_M1 = sample_bp['cpx_XAl_M1']**2 * cpx_W_MgAl_M1 + sample_bp['cpx_XFe3+_M1']**2 * cpx_W_MgFe_M1 + sample_bp['cpx_XAl_M1'] * sample_bp['cpx_XFe3+_M1'] * (cpx_W_MgAl_M1 + cpx_W_MgFe_M1 - cpx_W_FeAl_M1)
    
    cpx_diop_RTlna_TET = cpx_W_AlSi_M2 * ((sample_bp['cpx_Al_IV'] / 2)**2)
    
    cpx_diop_reciprocal = (-sample_bp['cpx_XAl_M1']) * sample_bp['cpx_XFe_M2'] * (-21000) - (sample_bp['cpx_XFe3+_M1'] * sample_bp['cpx_XFe_M2'] * (-4680)) + (sample_bp['cpx_XMg_M2'] * (-5010))
    # FAILED?
    cpx_diop_RTlna_CaMgSi = cpx_diop_RTln + cpx_diop_RTlna_M2 + cpx_diop_RTlna_M1 + cpx_diop_RTlna_TET + cpx_diop_reciprocal
                                                                                    
    cpx_CaTs_RTln = 8.314 * t * math.log(sample_bp['cpx_XAl_M1'] * (4 * sample_bp['cpx_XCa_M2']) * (sample_bp['cpx_Al_IV'] / 2) * (sample_wfu['cpx_Si'] / 2) )
    
    cpx_CaTs_RTlna_M2 = cpx_diop_RTlna_M2
    cpx_CaTs_RTlna_M1 = sample_bp['cpx_XMg_M1']**2 * cpx_W_MgAl_M1 + sample_bp['cpx_XFe3+_M1']**2 * cpx_W_FeAl_M1 + sample_bp['cpx_XMg_M1'] * sample_bp['cpx_XFe3+_M1'] * (cpx_W_MgAl_M1 +  cpx_W_FeAl_M1 - cpx_W_MgFe_M1)
    cpx_CaTs_RTlna_TET = cpx_W_AlSi_M2 * (sample_wfu['cpx_Si'] / 2)**2
    
    cpx_CaTs_reciprocal =  (1 - sample_bp['cpx_XAl_M1']) * sample_bp['cpx_XFe_M2'] * (-21000) - (sample_bp['cpx_XFe3+_M1'] * sample_bp['cpx_XFe_M2'] * (-4680)) + (sample_bp['cpx_XMg_M2'] * (-5010))
    # FAILED?
    
    cpx_CaTs_RTlna_CaAlAl = cpx_CaTs_RTln + cpx_CaTs_RTlna_M2 + cpx_CaTs_RTlna_M1 + cpx_CaTs_RTlna_TET + cpx_CaTs_reciprocal
    
#     print(cpx_diop_reciprocal)
#     print(cpx_CaTs_reciprocal)
#     print(cpx_CaTs_RTlna_CaAlAl)
    
    Eq_constant_RTlnK = cpx_CaTs_RTlna_CaAlAl + cpx_diop_RTlna_CaMgSi - ((1/3) * gt_pyp_Rtlna_MgAl) - ((2/3) *  gt_grs_Rtlna_CaAl)
    #About 800 off, but is this due to calculation error or differences in Fe3+?
    
    Eq_constant_K = math.exp(Eq_constant_RTlnK / (8.314 * t))
    # off by 0.16
   
    #Fitting parameters from Beyer (2015)
    delta_S = 23.5
    delta_H = 16500
    delta_Vr = 0.719
    
    delta_Gt = delta_H - delta_S * t
    
    pressure = (-(delta_Gt + 8.314 * t * math.log(Eq_constant_K))) / (delta_Vr * 1000)
    
    
    return pressure
    



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xenopt']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.2.0,<2.0.0']

setup_kwargs = {
    'name': 'xenopt',
    'version': '0.2.2',
    'description': 'A collection of published thermobarometers for garnet two-pyroxene xenoliths. Allows for input of .csv data, with template provided in tb_parameters.py',
    'long_description': '# XenoPT\nGarnet two-pyroxene thermobarometry package.\n\nhttps://pypi.org/project/xenopt/\n',
    'author': 'Khalil Droubi',
    'author_email': 'droubi@wisc.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/okdpetrology/XenoPT',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
